using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public interface IDragDropListener
{
    void OnEnemyPlaced(GameObject enemy, Opponent opponent, bool validPosition);
    void OnEnemyDeleted(GameObject enemy, Opponent opponent);
}

public class DragDestroyController : MonoBehaviour
{

    public static DragDestroyController instance { get; private set; }

    public List<IDragDropListener> listeners = new List<IDragDropListener>();

    [SerializeField] Color disabledColor;
    [SerializeField] LayerMask fighterRaycastLayer;
    [SerializeField] float fighterRaycastLength = 20f;
    [SerializeField] List<AudioClip> pickUpEnemySfx;
    [SerializeField] List<AudioClip> dropEnemySfx;
    [SerializeField] List<AudioClip> removeEnemySfx;

    bool draggingEnabled = true;
    GameObject selectedEnemy;
    SpriteRenderer selectedEnemySprite;
    Opponent selectedOpponent;
    Vector3 originalEnemyPosition;
    int originalEnemyLayer;

    int noCollisionLayer;
    BaseCameraController cameraController;

    public void HandoffEnemy(GameObject enemy, bool skipSound = true)
    {
        if (selectedEnemy != null)
        {
            PlaceSelectedEnemy();
        }
        selectedEnemy = enemy;
        originalEnemyPosition = enemy.transform.position;
        selectedEnemySprite = enemy.GetComponent<SpriteRenderer>();
        selectedOpponent = enemy.GetComponent<Enemy>().opponentType;
        originalEnemyLayer = enemy.layer;
        // So that raycasts don't hit the current Enemy
        enemy.layer = noCollisionLayer;
        if (!skipSound)
        {
            cameraController.PlayRandomGlobalAudioClip(pickUpEnemySfx);
        }
    }

    public void SetDraggingEnabled(bool enabled)
    {
        draggingEnabled = enabled;
    }

    void Update()
    {
        if (!RoundController.instance.roundRunning && draggingEnabled)
        {
            if (Input.GetButtonUp("Fire2") && selectedEnemy == null)
            {
                DeleteHoveredEnemy();
            }
            if (Input.GetButtonDown("Fire1"))
            {
                SelectHoveredEnemy();
            }
            if (Input.GetButtonUp("Fire1"))
            {
                PlaceSelectedEnemy();
            }
            MoveSelectedEnemy();
        }
    }

    GameObject RayCastForFighter(bool includeAntiHero)
    {
        Ray ray = Camera.main.ScreenPointToRay(Input.mousePosition);
        RaycastHit2D hit = Physics2D.GetRayIntersection(ray, fighterRaycastLength, fighterRaycastLayer);

        if (hit.collider == null || (!includeAntiHero && hit.collider.gameObject.CompareTag("AntiPlayer")))
        {
            return null;
        }

        return hit.collider.transform.gameObject;
    }

    void DeleteHoveredEnemy()
    {
        GameObject enemy = RayCastForFighter(false);

        if (enemy != null)
        {
            HandoffEnemy(enemy);
            listeners.ForEach(l => l.OnEnemyDeleted(enemy, selectedOpponent));
            Destroy(enemy);
            cameraController.PlayRandomGlobalAudioClip(removeEnemySfx);
        }
    }

    void SelectHoveredEnemy()
    {
        GameObject enemy = RayCastForFighter(false);
        if (enemy != null)
        {
            HandoffEnemy(enemy, false);
        }
    }

    void MoveSelectedEnemy()
    {
        if (selectedEnemy != null)
        {
            // Move Enemy to mouse position
            selectedEnemy.transform.position = GetMouseWorldPosition() - (Vector3)selectedOpponent.dragPivot;

            // Indicate that Enemy can't be placed on top of other Enemies
            selectedEnemySprite.color = RayCastForFighter(true) != null ? disabledColor : Color.white;
        }
    }

    void PlaceSelectedEnemy()
    {
        if (selectedEnemy != null)
        {
            var validPosition = RayCastForFighter(true) == null;
            if (!validPosition)
            {
                // When trying to place the Enemy on another Fighter, reset it to its original position
                selectedEnemy.transform.position = originalEnemyPosition;
                selectedEnemySprite.color = Color.white;

            }
            selectedEnemy.layer = originalEnemyLayer;
            listeners.ForEach(l => l.OnEnemyPlaced(selectedEnemy, selectedOpponent, validPosition));
            selectedEnemy = null;
            cameraController.PlayRandomGlobalAudioClip(dropEnemySfx);
        }
    }

    Vector3 GetMouseWorldPosition()
    {
        Vector3 pos = Camera.main.ScreenToWorldPoint(Input.mousePosition);
        pos.z = selectedEnemy.transform.position.z;
        return pos;
    }

    void Awake()
    {
        if (instance == null)
        {
            instance = this;
        }
        else if (instance != this)
        {
            Destroy(gameObject);
        }
        DontDestroyOnLoad(gameObject);

        noCollisionLayer = LayerMask.NameToLayer("No Collision");
        cameraController = Camera.main.GetComponent<BaseCameraController>();
    }
}
